#!/usr/bin/perl

use strict;
use warnings;

print STDOUT "\n\n";
print STDOUT "===================================================================================\n";
print STDOUT "| Partial HGT-DETECTION V.1.2 (June, 2013) by Alix Boc and Vladimir Makarenkov    |\n"; 
print STDOUT "===================================================================================\n";

#======================================================
#= VERIFICATION DES ARGUMENTS DE LA LIGNE DE COMMANDE
#======================================================
if( scalar @ARGV < 1){
    print "Erreur\nusage : $0 speciesTreeFile=[Species tree file] geneSequencesFile=[Gene sequences file] opt_m=[PhyML|NJ] opt_ws=[100] opt_ss=[10] opt_st=[DNA|RNA|AA] opt_nr=[10] opt_bm=[60] opt_ncw=[3]\n";
    print "\nspeciesTreeFile\tSpecies tree file name. The species tree should be in the Newick format";
	print "\ngeneSequencesFile\tGene sequences file name. The sequences should be in the Phylip format";
	print "\nopt_m \tTree inference (PhyML or NJ)                                    : default NJ"; 
	print "\nopt_ws\tSliding window size                                             : default=100";
    print "\nopt_ss\tStep size                                                       : default=10";
    print "\nopt_st\tSequence type (DNA,AA)                                          : default=AA";
	print "\nopt_nr\tNumber of replicates in bootstrap                               : default=10";
    print "\nopt_bm\tDisplay transfers with bootstrap support higher than x%         : default=60%";
    print "\nopt_ncw\tNumber of consecutive windows necessary to validate a transfer  : default=3";
	print "\n";
	exit 0;
}

#clean();
#============================================================================
#= Gestion des parametres = valeurs par defaut
#============================================================================
my $fenetre 		 	 	= 100;		#= taille de la fenetre
my $increment 		  		= 10;		#= deplacement de la fenetre
my $nbBootstrap 	  		= 10;		#= nombre d'arbre pour le bootstrap
my $boot_min 		  		= 60;		#= bootstrap minimum acceptable
my $pos_debut_globale 		= 0;		#= position de debut
my $reconstruction	  		= "NJ";	    #= NJ,ML
my $sequence_type 	  		= "AA";		#= DNA,AA
my $speciesTreeFile			= "";
my $geneSequencesFile		= "";
my $min_hgt_length          = 0;
my $ncw						= 3;
#===========================================================================
#= Gestion des parametres = nouvelle valeurs
#===========================================================================
my $date_debut = `date`;
my $param;
my @tab_tmp;

for(my $i=0;$i<scalar @ARGV;$i++){	
	$param =  $ARGV[$i];		
	chomp($param);	
	@tab_tmp = split("=",$param);
	if($tab_tmp[0] eq "opt_st"){	
		$sequence_type = $tab_tmp[1];	
		if(($sequence_type ne "DNA") && ($sequence_type ne "AA")){
			print STDOUT "$sequence_type : Unknown sequence type (DNA,AA)\n"; 
			exit;
		}
	}
	elsif($tab_tmp[0] eq "opt_ws"){	
		$fenetre = $tab_tmp[1];
		$min_hgt_length = $fenetre;
	}
	elsif($tab_tmp[0] eq "opt_ncw"){	
		$ncw = $tab_tmp[1];
	}
	elsif($tab_tmp[0] eq "opt_m"){	
		$reconstruction = $tab_tmp[1];
	}
	elsif($tab_tmp[0] eq "opt_ss"){	
		$increment = $tab_tmp[1];
	}
	elsif($tab_tmp[0] eq "opt_bm"){	
		$boot_min = $tab_tmp[1];
	}
	elsif($tab_tmp[0] eq "opt_nr"){
		$nbBootstrap = $tab_tmp[1];
	}
	elsif($tab_tmp[0] eq "speciesTreeFile"){
		$speciesTreeFile = $tab_tmp[1];
	}
	elsif($tab_tmp[0] eq "geneSequencesFile"){
		$geneSequencesFile = $tab_tmp[1];
	}
	else{	
		print STDOUT $tab_tmp[0] . ": Unknown parameters\n"; 
		exit;	
	}	
}

$min_hgt_length = $fenetre + ($ncw-1)*$increment;

#============================================================
#= Affichage des donnes de simulation
#============================================================
print STDOUT "Species Tree filename   = $speciesTreeFile\n";
print STDOUT "Gene Sequences filename = $geneSequencesFile\n";
print STDOUT "Sliding windows size    = $fenetre\n";
print STDOUT "Step size               = $increment\n";
print STDOUT "Number of replicates    = $nbBootstrap\n";
print STDOUT "Minimum Bootstrap       = $boot_min\n";
print STDOUT "Tree reconstruction     = $reconstruction\n";
print STDOUT "Sequence Type           = $sequence_type\n";
print STDOUT "Number of consecutive windows necessary to validate a transfer = $ncw\n";

open (OUT,">phgt_output.txt");
print OUT "===================================================================================\n";
print OUT "| Partial HGT-DETECTION V.1.2 (June, 2013) by Alix Boc and Vladimir Makarenkov |\n"; 
print OUT "===================================================================================\n";
print OUT "Species Tree filename   = $speciesTreeFile\n";
print OUT "Gene Sequences filename = $geneSequencesFile\n";
print OUT "Sliding windows size    = $fenetre\n";
print OUT "Step size               = $increment\n";
print OUT "Number of replicates    = $nbBootstrap\n";
print OUT "Minimum Bootstrap       = $boot_min\n";
print OUT "Tree reconstruction     = $reconstruction\n";
print OUT "Sequence Type           = $sequence_type\n";
print OUT "Number of consecutive windows necessary to validate a transfer = $ncw\n";
close (OUT);

my $phyml 		= "exec/phyml";
my $seqboot		= "exec/seqboot";
my $neighbor	= "exec/neighbor";
my $rf			= "exec/rf";
my $dnadist     = "exec/dnadist";
my $protdist    = "exec/protdist";
my $consense	= "exec/consense";

#=============================================================================================
#= PARTIAL HGT-DETECTION 
#=============================================================================================
my $speciesTreeNewick;				
my %geneSequences 		= ();
my $taille_sequence 	= 0;
my $nombre_especes 		= 0;
my $tmp;
my $hgtFoundFile 		= "hgtdetectes.txt";
my $firstOutputWeb 		= "firstOutputWeb.txt";
my $infile 				= "infile";
my $outfile 			= "outfile";
my $outtree 			= "outtree";
my $intree              = "intree";	
my $inputHgtFile 		= "input.txt";	
my $outputHgtFile 		= "output.txt";		
my $seqbootConf 		= "seqbootConf.txt";	#= fichier de config de seqboot
my $dnadistConf 		= "dnadistConf.txt";	#= fichier de config de dnadist
my $protdistConf 		= "protdistConf.txt";	#= fichier de config de protdist
my $neighborConf 		= "neighborConf.txt";	#= fichier de config de neighbor
my $phymlConf 			= "phymlConf.txt";		#= fichier de config de phyml 
my $seed 				= 3;					#= seed pour les progs Phylip
my $log 				= "log.txt";			#= fichier log
my $pos_fin;
my %transferts 			= ();
my %transferts2 		= ();
my $pos_debut 			= $pos_debut_globale;
my $bootTree 			= 0;
my $RF_distance			=-1;
my $bootTreeFile		= "bootTreeFile.txt";
my $consenseConf		= "consenseConf.txt";
my $convertionNameFile 	= "convertNameFile.txt";
my $nbNan               = 0;
my $noWindow			= 0;
my $templateOutputWeb   = "";

if( -e "$outputHgtFile.tmp"){
	execute("rm -rf $outputHgtFile.tmp");
}

#======================================================
#= Conversion des noms d'especes en a_%_a
#======================================================
my %nom_conv = ();
my $cpt_species = 0;

#print STDOUT "\nArbre initial : " . join("",file_get_contents($speciesTreeFile));

execute("cp $geneSequencesFile user_geneSequencesFile.phy");
execute("cp $speciesTreeFile user_speciesTreeFile.new");

$geneSequencesFile = "user_geneSequencesFile.phy";
$speciesTreeFile   = "user_speciesTreeFile.new";

%geneSequences = chargerSequence($geneSequencesFile);
$nombre_especes = scalar keys(%geneSequences);
@tab_tmp = keys(%geneSequences);
$taille_sequence = length($geneSequences{$tab_tmp[0]});

open (IN,$speciesTreeFile) || goto ERROR;
my $speciesTree = <IN>;
close (IN);

open (OUT,">$geneSequencesFile") || goto ERROR;
open OUT_CONV , ">$convertionNameFile" || goto ERROR;
print OUT $nombre_especes . "  " . $taille_sequence;
print OUT_CONV "\n#internal species names,real species names"; 
foreach my $key (keys %geneSequences){
	$cpt_species++;
	my $a_nm_a = "a_" . $cpt_species . "_a";
	print OUT "\n$a_nm_a " . $geneSequences{$key} ;
	$speciesTree =~ s/$key/$a_nm_a/g;
	$nom_conv{$a_nm_a} = $key;
	print OUT_CONV "\n$a_nm_a,$key";
}
close (OUT_CONV);
close (OUT);

open (OUT,">$speciesTreeFile") || goto ERROR;
print OUT $speciesTree;
close(OUT);




#print STDOUT "\n\nArbre transforme : " . join("",file_get_contents($speciesTreeFile));



#======================================================
#= LECTURE DES DONNEES (arbre et sequences)
#======================================================
@tab_tmp = file_get_contents($speciesTreeFile);
$speciesTreeNewick = $tab_tmp[0];

%geneSequences = chargerSequence($geneSequencesFile);
$nombre_especes = scalar keys(%geneSequences);
@tab_tmp = keys(%geneSequences);
$taille_sequence = length($geneSequences{$tab_tmp[0]});


#=====================================================
#= Ajustement de la taille de la fenetre coulissante
#=====================================================
if($taille_sequence <= $fenetre){
	$fenetre = $taille_sequence - 1;
	$min_hgt_length = $fenetre + ($ncw-1)*$increment;
}

#======================================================
#= INITIALISATION DES FICHIERS DE CONFIGURATION
#======================================================
execute("echo \"R\n$nbBootstrap\ny\n$seed\n\" > $seqbootConf");
execute("echo \"M\nd\n$nbBootstrap\ny\n\" > $dnadistConf");
execute("echo \"M\n$nbBootstrap\n$seed\ny\n\" > $neighborConf");
execute("echo \"M\nD\n$nbBootstrap\np\np\np\ny\n\" > $protdistConf");
execute("echo \"y\n\" > $consenseConf");

if($sequence_type eq "DNA"){
	execute("echo \"infile\nb\n$nbBootstrap\ny\ny\n\" > $phymlConf");
	execute("echo \"$infile\n+\n+\nO\nL\n+\nb\n$nbBootstrap\ny\ny\n\" > $phymlConf");
}
else{
	#execute("echo \"$infile\nd\n+\nM\nM\nM\n+\nO\nL\n+\nb\n$nbBootstrap\ny\nY\n\" > $phymlConf");
	execute("echo \"$infile\nd\n+\nM\nM\nM\n+\nS\nS\nS\n+\nb\n$nbBootstrap\ny\nY\n\" > $phymlConf");
}

my $last = 0;
my $zone = 0;
my $nouvelle_zone = "yes";
my $iteration = 0;
my $nbLigne   = 0;
FOO:
while ( ($pos_debut < $taille_sequence) && ($last == 0)){	
	if(($pos_debut + $fenetre) > ($taille_sequence-1)){
		$pos_debut = $taille_sequence - 1 - $fenetre;
		$last = 1;
	}
	
	if ($nbLigne % 50 == 0){
		print STDOUT "\n=================================================================================================================";
		print STDOUT "\n|    Interval     |   RF  | Bootstrap | Detected transfers";
		print STDOUT "\n=================================================================================================================";
	}
	$nbLigne++;
	
	printf (STDOUT "\n| [%5d - %5d]",$pos_debut,$pos_debut + $fenetre);
	
	$pos_fin = $pos_debut + $fenetre;
	#======================================================
	#= CREATION DU FICHIER INPUT DE SEQUENCES
	#======================================================
	open(OUT,">$infile") || goto ERROR;
	print OUT "$nombre_especes $fenetre";
	foreach my $elt( keys(%geneSequences)){
		my $espace;
		for(my $i=1;$i<=(10-length($elt));$i++){
			$espace .= " ";
		}
		print OUT "\n" . $elt ."$espace". substr($geneSequences{$elt},$pos_debut,$fenetre);
	}
	close(OUT);
	
	execute("cat $speciesTreeFile > $inputHgtFile");
	
	#======================================================
	#= GENERATION DES ARBRES
	#======================================================
	if($reconstruction eq "NJ"){	
		execute("rm -rf $outfile >> $log");
		#print STDOUT " seqboot..";
		execute("$seqboot < $seqbootConf >> $log");		#= generation des autres arbres
		
		#`cat $inputHgtFile > $infile`; 			#= copie de l'arbre initial
		#`echo "" >> $infile`; 	
		execute("cat $outfile > $infile");				#= ajout des autres arbres
		execute("rm -rf $outfile >> $log");
		if($sequence_type eq "DNA"){
			#print STDOUT "dnadist..");
			execute("$dnadist < $dnadistConf >> $log");		#= calcul des matrices de distances	
			if(!-e $outfile){
				print STDOUT "\nProblems with dnadist...\n";
				exit;	
			}
			execute("mv $outfile $infile");
			execute("rm -rf $outtree >> $log");
		}
		else{
			#print STDOUT "protdist..";
			execute("$protdist < $protdistConf >> $log");		#= calcul des matrices de distances	
			execute("mv $outfile $infile");
			execute("rm -rf $outtree >> $log");
		}
		#print STDOUT "neighbor..";
		
		$nbNan = `grep "nan" $infile | wc -l`; 
		
		
		#== Suppression des valeurs non calculables dans la matrice (nan->100000)
		if ($nbNan != 0){
			my @tab_infile = file_get_contents($infile);
			open OUT_INFILE , ">$infile" || die "$!";
			foreach my $ligne (@tab_infile){
				$ligne =~ s/nan/10000/g;
				print OUT_INFILE $ligne;
			}
			close OUT_INFILE;
		}
		
		$nbNan = `grep "nan" $infile | wc -l`; 
	
		if($nbNan == 0){
		  #  print STDOUT "\nnbNan=$nbNan";
			execute("$neighbor < $neighborConf >> $log");	#= transformation en chaine newick
    		execute("cp $outtree $intree");		
    		
    		execute("echo \"$speciesTreeNewick\" > $inputHgtFile");
    		
    		execute("tr -d '\n\r' < $outtree | sed 's/;/;\\\n/g' | sed 's/-//g' >> $inputHgtFile");
    		
    		execute("rm -rf $outtree $outfile");
    		#execute("mv intree_save intree");
    		execute("$consense < $consenseConf >> $log");	
    		execute("grep 'Sets in' -A 100 $outfile | grep 'Sets NOT' -B 100 | grep '\*' | grep \"[0-9][0-9]*.[0-9][0-9]*\" -o > $bootTreeFile");
    		
    		my @tab_tmp=file_get_contents("$bootTreeFile");
    		
    		$bootTree=0;
    		for(my $i=0;$i<scalar @tab_tmp;$i++){
    			$bootTree += ($tab_tmp[$i]*100)/($nbBootstrap+1);	
    		}
    		$bootTree /= scalar @tab_tmp;
    		
    		@tab_tmp = file_get_contents2("$inputHgtFile");
    		my $arbre2=$tab_tmp[1];
    		
    	    #print STDOUT "$speciesTreeNewick\n$arbre2\n";
    		$RF_distance = robinson_and_foulds($speciesTreeNewick,$arbre2);
    		
    		#print STDOUT "$speciesTreeNewick\n$arbre2\n";
    		
    		printf( STDOUT " |  %3d  |    %3.0lf%%   | ",$RF_distance,$bootTree);
		}
		else{
		  printf( STDOUT " |  Distance matrices cannot be build");
		  
		}
	}
	else{
		if($reconstruction eq "PhyML"){
			my $cmd = "rm -rf $infile" . "_*";
			execute("$cmd");
			#print STDOUT " phyml..";
			execute("$phyml < $phymlConf >> $log");
			my @tab_tmp = file_get_contents("$infile" . "_phyml_tree.txt");
			my $chaine = $tab_tmp[0];
			(my @tab_boot) = ($chaine =~ /\)([0-9][0-9]*):/g);
			#print STDOUT $chaine;
			my $total = 0;
			my $boot_cpt=0;
			foreach my $val (@tab_boot){
				$total += $val;
				$boot_cpt++;
			}
	
			$bootTree = (($total/$boot_cpt)*100)/$nbBootstrap;
			
			$chaine =~ s/\)[0-9][0-9]*:/\):/g;
			
			@tab_tmp = file_get_contents("$speciesTreeFile");
			$RF_distance = robinson_and_foulds($tab_tmp[0],$chaine);
			printf( STDOUT " |  %3d  |    %3.0lf%%   |",$RF_distance,$bootTree);
			
			$cmd = "echo \"\n$chaine\" >> $inputHgtFile";
			execute("$cmd");
			@tab_tmp = file_get_contents("$infile" . "_phyml_boot_trees.txt");
			
			foreach $chaine (@tab_tmp){
				
				chomp ($chaine);
				$cmd = "echo \"\n$chaine\" >> $inputHgtFile";
				execute("$cmd");
			
			}
		}
	}
	
	#======================================================
	#= DETECTION DES TRANSFERTS AVEC BOOTSTRAP
	#======================================================	
	#print STDOUT "hgt-detection..";
	#execute("sed 's/0\.000000/0\.00005/g' > $inputHgtFile");	
	
	if(($RF_distance > 1) && ($bootTree >= 0) && ($nbNan == 0 ))
	{				
		if($nouvelle_zone eq "yes"){
			$zone++;
		}
		$nouvelle_zone = "no";
		$iteration++;
		execute("rm -rf output.txt outputWeb.txt results.txt nomorehgt.txt log_hgt.txt return.txt");
		#execute("perl run_hgt.pl -inputfile=$inputHgtFile -bootstrap=yes -speciesroot=file -generoot=file >> $log"); #
		execute("perl run_hgt_int.pl -inputfile=$inputHgtFile -bootstrap=yes -speciesroot=midpoint -generoot=midpoint >> $log"); #
	#	print STDOUT "perl run_hgt.pl -inputfile=$inputHgtFile -bootstrap=yes";
		$noWindow++;
		#print STDOUT "\nAPRES EXECUTION DE HGT";
		if( -e "outputWeb.txt"){
			if($templateOutputWeb eq ""){
				$templateOutputWeb = "outputWeb_$noWindow.txt";
				#print STDOUT "\ntemplateOutputWeb = $templateOutputWeb";
			}

			execute("mv $firstOutputWeb outputWeb_$noWindow.txt");
			
			#print STDOUT "\nmv outputWeb.txt outputWeb_$noWindow.txt";;
			execute("cat $inputHgtFile >> $log");
			execute("cat inputfileformated.txt >> $log");
			execute("cat $outputHgtFile >> $log");
			#execute("cat log_hgt.txt >> $log");
		
			#======================================================
			#= LECTURE DES RESULTATS
			#======================================================
			#if( -e $outputHgtFile)
			{
				
			my @tab_output = file_get_contents($outputHgtFile);
			
			open(OUT,">>$outputHgtFile.tmp") || goto ERROR;
			my @tab_list_dest=();
			my @tmp_tab;
			
			my $listeBranches = `grep "hgt = " outputWeb_$noWindow.txt`;		
			$listeBranches =~ s/hgt = //g;
			chomp($listeBranches);
			my @tab_listeBranches = split(",",$listeBranches);
		#	print STDOUT "\nListe des branches : $listeBranches";
			my $ib=0;
			my %verife = ();
			open(IN,"$outputHgtFile") || die "Cannot open $outputHgtFile ($!)";
			foreach my $ligne (@tab_output){
				chomp($ligne);
				$ligne =~ s/ //g;
				print STDOUT "*";
				#print STDOUT "\nTransfert : $ligne";
				(my $source,my $dest,my $bootstrap) = split("<>",$ligne);
				@tmp_tab    = split(",",$source);
				$source     = join(",",@tmp_tab); #join(",",sort par_num @tmp_tab);
				my @tab_source = split(",",$source);
				@tmp_tab    = split(",",$dest);
				$dest       = join(",",@tmp_tab); #join(",",sort par_num @tmp_tab);
				my @tab_dest  =	split(",",$dest);
				
				my $a1 = $tab_listeBranches[$ib++];				
				my $a2 = $tab_listeBranches[$ib++];				
				my $b1 = $tab_listeBranches[$ib++];				
				my $b2 = $tab_listeBranches[$ib++];				
				
				#print STDOUT "\nBranche : $a1,$a2,$b1,$b2";
				#print STDOUT "\nbootstrap=$bootstrap,boot_min=$boot_min";	
				if($bootstrap > $boot_min){
					print OUT "$iteration<>$RF_distance<>$pos_debut<>$pos_fin<>$source<>$dest<>$bootstrap<>$a1<>$a2<>$b1<>$b2<>end\n";
				}
			}
			close(IN);
			close(OUT);
			}
			
		#	if($noWindow == 3){
		#		exit;
		#	}
		}
	}
	else{
		$nouvelle_zone = "yes";
	}	
	if($RF_distance < 2){
		$pos_debut += 2*$increment;
	}
	
	if(($pos_debut + $fenetre) == ($taille_sequence-1)){
		$pos_debut = $taille_sequence;
	}

	$pos_debut = $pos_debut + $increment;
}

print STDOUT "\n=================================================================================================================\n";
open (OUT,">>phgt_output.txt");
print OUT "\n\nPartial HGT detected with bootstrap support higher than $boot_min% : \n";
print OUT "================================================================";

{
	my $max_rf = -1;
	execute("cat $outputHgtFile.tmp  > $outputHgtFile");
	open(IN,"$outputHgtFile") || goto ERROR;
	
	while(my $ligne = <IN>){
		chomp($ligne);
		if($ligne =~ /<>/){
			$ligne =~ s/ //g;
			(my $iteration,my $RF,$pos_debut,$pos_fin,my $source,my $dest,my $bootstrap,my $a1,my $a2,my $b1,my $b2,my $end) = split("<>",$ligne);
			 if($max_rf == -1){
				$max_rf = $RF;
			}
			{
				my @tmp_tab = split(",",$source);
				$source = join(",",@tmp_tab); #join(",",sort par_num @tmp_tab);
				@tmp_tab = split(",",$dest);
				$dest = join(",",@tmp_tab); #join(",",sort par_num @tmp_tab);
				if($bootstrap > $boot_min){
					my @tmp_tab2 = split("<>",$ligne);
					print OUT "\n\nTransfer  : " . $tmp_tab2[4] ."->". $tmp_tab2[5];
		#			print OUT "\nBranches    : $a1,$a2 -> $b1,$b2";
					printf( OUT "\nBootstrap : %1.1lf%%", $tmp_tab2[6]);
					print OUT "\nInterval  : " . ($tmp_tab2[2]+1) . " : " . ($tmp_tab2[3]+1);
					
					if(exists($transferts{"$source->$dest"} )){
						for(my $i=$pos_debut;$i<=$pos_fin;$i++){
							$transferts{"$source->$dest"}[$i] = 1;
						}
					}
					else{
						for(my $i=0;$i<$taille_sequence;$i++){
							$transferts{"$source->$dest"}[$i] = 0;
						}
						for(my $i=$pos_debut;$i<=$pos_fin;$i++){
							$transferts{"$source->$dest"}[$i] = 1;
						}
					}
				}
			}
		}
	}
	close(IN);
}

#======================================================
#= AFFICHAGE DES RESULTATS
#======================================================
print OUT "\n\n\nOverlapping partial HGT detected with bootstrap support higher than $boot_min% and validated over $ncw consecutive windows\n";
print OUT "====================================================================================================================";

my %results  = ();
my %compteur = ();
my %interval_debut = ();
my %interval_fin = ();
my %branches = ();

my $l_hgt       = "";
my $l_valide    = "";
my $l_sequence  = "";
my $l_bootstrap = "";

open (IN,$outputHgtFile) || die "Impossible d'ouvrir le fichier $outputHgtFile ($!)";
while( my $ligne=<IN>){
	
	$ligne =~ s/\[//g;
	$ligne =~ s/\]//g;
	my @tmp = split("<>",$ligne);
	my $key = $tmp[4] . "->" . $tmp[5];
	
	$branches{$key} = $tmp[7] . "," . $tmp[8] . "," . $tmp[9] . "," . $tmp[10] ; 
	
	if(exists($results{$key})){
		
		if($tmp[6] >= ($boot_min/2)){
			if( ($tmp[2] <= $interval_fin{$key} ) && ($tmp[2] > $interval_debut{$key} ) ){
				$interval_fin{$key} = $tmp[3];
			}
			if( ($tmp[2] > $interval_fin{$key} ) && (($tmp[3]-$interval_fin{$key}) < 40) ){
				$interval_fin{$key} = $tmp[3];
			}
			$results{$key}  = $results{$key} + $tmp[6];
			$compteur{$key} = $compteur{$key} + 1;
		}
	}
	else{
		$results{$key}  = $tmp[6];
		$compteur{$key} = 1;
		$interval_debut{$key}  = $tmp[2];
		$interval_fin{$key}  = $tmp[3];
	}
	
}

my $virg = "";
foreach my $key (keys %results){
	if(($results{$key}/$compteur{$key} > $boot_min) && ( ($interval_fin{$key}-$interval_debut{$key}) >= $min_hgt_length)){
		print OUT "\n\nTransfer          : $key";
#		print OUT "\nbranches          : " . $branches{$key};
		printf( OUT "\nAverage bootstrap : %1.1lf%%", $results{$key}/$compteur{$key});
		print OUT "\nInterval          : " . ($interval_debut{$key}+1) ."-". ($interval_fin{$key}+1);

		$l_hgt       .= $virg . "" . $branches{$key};
		$l_valide    .= $virg . "" . "1";
		$l_sequence  .= $virg . "" . "1";
		$l_bootstrap .= $virg . "" . $results{$key}/$compteur{$key}. "<>" . ($interval_debut{$key}+1) ." : ". ($interval_fin{$key}+1);
		$virg=",";
	}
}
#print OUT "\n\n";
#print OUT "noWindow = $noWindow\n";
#print OUT "hgt = $l_hgt\n";
#print OUT "hgt_reels = $l_hgt\n";
#print OUT "hgt_valide = $l_valide\n";
#print OUT "hgt_sequence = $l_sequence\n";
#print OUT "hgt_bootstrap = $l_bootstrap\n";
close (OUT);

open (IN,$templateOutputWeb) || goto ERROR;
open (OUT,">outputWeb.txt")  || goto ERROR;
while (my $ligne = <IN>){
	chomp($ligne);
	if($ligne =~ /^hgt = /){
		print OUT "hgt = $l_hgt\n" if($l_hgt ne "");
	}
	elsif($ligne =~ /^hgt_reels = /){
		print OUT "hgt_reels = $l_hgt\n" if($l_hgt ne "");
	}
	elsif($ligne =~ /^hgt_valide = /){
		print OUT "hgt_valide = $l_valide\n" if($l_hgt ne "");
	}
	elsif($ligne =~ /^hgt_sequence = /){
		print OUT "hgt_sequence = $l_sequence\n" if($l_hgt ne "");
	}
	elsif($ligne =~ /^hgt_bootstrap = /){
		print OUT "hgt_bootstrap = $l_bootstrap\n" if($l_hgt ne "");
	}
	else{
		print OUT "$ligne\n";
	}
}
print OUT "bootHGT = $l_bootstrap" if($l_hgt ne "");

close(OUT);
close(IN);


open(OUT,">$hgtFoundFile") || goto ERROR;
my $saut = "";

my $pred = 0;

foreach my $elt (keys %transferts){
	for(my $i=0;$i<$taille_sequence;$i++){		
		if(($transferts{$elt}[$i] == 1) && ($pred == 0)){
			$saut = "\n";
			$pred = 1;
		}
		elsif((($transferts{$elt}[$i] == 0)||($i == ($taille_sequence-1))) && ($pred == 1)){
			$pred=0;
		}
	}
}

close(OUT);

clean();


#================================================
#= Retablissement des bons noms
#================================================

my $output_content = "";
my $web_content    = "";

open (IN,"phgt_output.txt") || goto ERROR;
while (my $ligne = <IN>){
	foreach my $key (keys %nom_conv){
		my $value = $nom_conv{$key};
		$ligne =~ s/$key/$value/g;
	}
	$output_content .= $ligne;
}
close (IN);

open (IN,"outputWeb.txt") || goto ERROR;
while (my $ligne = <IN>){
	foreach my $key (keys %nom_conv){
		my $value = $nom_conv{$key};
		$ligne =~ s/$key/$value/g;
	}
	$web_content .= $ligne;
}
close (IN);

#print STDOUT $output_content;

execute ("echo \"$output_content\" > phgt_output.txt");
execute ("echo \"$web_content\" > outputWeb.txt");


execute("mv phgt_output.txt output.txt");
execute("rm -rf outputWeb.txt") if($l_hgt eq "");
execute("echo \"0\" > return.txt");


#=========================== envoie du mail =============================
envoie_mail("ok");
print STDOUT "\nSee results in the file output.txt\n\nEnd of the computation ....\n";
#clean();
exit;


ERROR:
envoie_mail("error");
print STDOUT "\nSee results in the file output.txt\n\nEnd of the computation ....\n";
#clean();
exit;


#=====================================================================================================
#= SOUS PROGRAMMES
#=====================================================================================================
sub envoie_mail{
	if(-f "cmdrc"){
		my $status = $_[0];
		my $date_fin = `date`;
		print STDOUT "\nenvoie du mail";

		my $mail = "";
		my $rep_mail = "";
		my $subject = "Trex : Result for Partial HGT-Detection";
	
		open IN , "cmdrc" || die ($!);
		while( my $ligne = <IN>){
			chomp($ligne);
			if($ligne =~ /mail/){
				($tmp,$mail) = split("=",$ligne);
			}
			if($ligne =~ /rep_mail/){
				($tmp,$rep_mail) = split("=",$ligne);
			}
		}
		close IN;

		open OUT, ">message_mail.txt" || die ($!);
		print OUT "Results for Partial HGT-Detection\n\nFollow the link below to access the results of your computation (the data will be available for 7 days from now) :";
		print OUT "\nhttp://www.trex.uqam.ca/loadData.php?actionTrex=hgtpartialresult&id=$rep_mail&status=$status";
		print OUT "\n\nComputation started at: $date_debut";
		print OUT "Computation ended at:  $date_fin";
		print OUT "\n\n The T-REX Web Server team";
		close OUT;
		execute("/usr/bin/php /home/trex/public_html/scripts/sendmail.php $mail message_mail.txt");
	}
}


sub delete_elt_array{
	(my $elt,my @tab) = @_;
	my @new_tab=();
	foreach my $val (@tab){
		push(@new_tab,$val) if($val ne $elt);	
	}
	return @new_tab;
}

sub est_detecte{
	my $trans_det = $_[0]."->".$_[1];
	my $fichier= $_[2];
	#======================================================
	#= LECTURE DES TRANSFERTS SIMULES
	#======================================================
	open (IN,$fichier) || die "Impossible d'ouvrir le fichier $fichier ($!)";
	my @hgt_simules = <IN>;
	close(IN);	
	for(my $i=0;$i<scalar @hgt_simules;$i++){	
		my $ligne1 = $hgt_simules[$i];
		chomp($ligne1);
		(my $trans_sim,my $sd,my $sf) = split("<>",$ligne1);
		if($trans_sim eq $trans_det){
			return 1;
		}
	}
	return 0;
}

sub robinson_and_foulds{
	file_put_contents("rf_input.txt",$_[0] . "\n" . $_[1]);
	execute("$rf rf_input.txt rf_output.txt rf_tmp.txt  rf_output.tmp rf_matrices.txt ");
	my @tab_tmp = file_get_contents("rf_output.txt");
	(my $RF) = ($tab_tmp[5] =~ /.*=\ ([0-9][0-9]*)/);
	return $RF;
}

sub file_get_contents{
	open(IN,$_[0]) || die "Impossible d'ouvrir " . $_[0] . " ($!)";
	my $i=0;
	my @return = <IN>;
	close(IN);
	return @return;
}

sub file_get_contents2{
	open(IN,$_[0]) || die "Impossible d'ouvrir " . $_[0] . " ($!)";
	my $i=0;
	my @return = (); 
	while(my $ligne =<IN>){
	   if( $ligne =~ ";"){
	       $return[$i] = $ligne;
	       $i++;
	   }
	}
	close(IN);
	return @return;
}

sub file_put_contents{
	my $fichier = $_[0];
	open(OUT,">$fichier") || die "Impossible d'ouvrir $fichier ($!)";
	print OUT $_[1];
	close(OUT);
}

sub execute{
	my $cmd = $_[0];
	my $retour = system("$cmd");
}
sub writeInFile{
	(my $contenu, my $fichier) = @_;
	open (OUT,">$fichier") || die "impossible d'ouvrir $fichier ($!)";
	print OUT "$contenu";
	close (OUT);
}

#==========================================================================
#==
#==========================================================================
sub chargerSequence{
	my $file = $_[0];
	my $tmp=0;
	my %geneSequences = ();
	open(IN,$file) || die "Cannot open $file ($!)";
	while (my $ligne = <IN>){
			chomp($ligne);
			if($tmp > 0){
				(my $name, my $sequence) = split(" ",$ligne);
				$geneSequences{$name} = $sequence;
			}
			else{
				#($nombre_especes,$taille_sequence) = split(" ",$ligne);
			}			
			$tmp ++;
	}
	close (IN);
	return %geneSequences;
}

sub clean{
    execute("rm -rf outputWeb_* bootTreeFile.txt infile_phyml_boot_stats.txt input.txt nomorehgt.txt phymlConf.txt rf_input.txt speciesRootLeaves.txt tmp_input.txt");
    execute("rm -rf consenseConf.txt geneRootLeaves.txt infile_phyml_boot_trees.txt protdistConf.txt rf_output.tmp speciesRootTmp.txt");
    execute("rm -rf geneRootTmp.txt infile_phyml_stats.txt intree output.txt.tmp rf_output.txt speciesRoot.txt");
    execute("rm -rf firstResults* user_* geneRoot.txt infile_phyml_tree.txt results2.txt rf_tmp.txt");
    execute("rm -rf dnadistConf.txt hgtdetectes.txt inputfileformated.txt results.txt sync_projet.sh");
    execute("rm -rf errorFile.txt infile input_.txt neighborConf.txt return.txt seqbootConf.txt test.txt outtree outfile");
}

